/* Copyright (c) Daniel Newman 2006
 *
 * e-mail: danielnewman106@hotmail.com
 */


// GameState.cpp: implementation of the CGameState class.
//
//////////////////////////////////////////////////////////////////////

#include "GameState.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGameState::CGameState() : m_bGoatsTurn(true)
{
	Reset ();
}

CGameState::~CGameState()
{

}

bool CGameState::CanMoveDownLeft (int X, int Y)
{
	switch (X)
	{
	case 1: case 3: return Y == 1 || Y == 3;
	case 2: case 4: return Y == 0 || Y == 2;
	default: return false;
	}
}

bool CGameState::CanMoveDownRight (int X, int Y)
{
	switch (X)
	{
	case 0: case 2: return Y == 0 || Y == 2;
	case 1: case 3: return Y == 1 || Y == 3;
	default: return false;
	}
}

bool CGameState::CanMoveUpLeft (int X, int Y)
{
	switch (X)
	{
	case 1: case 3: return Y == 1 || Y == 3;
	case 2: case 4: return Y == 2 || Y == 4;
	default: return false;
	}
}

bool CGameState::CanMoveUpRight (int X, int Y)
{
	switch (X)
	{
	case 0: case 2: return Y == 2 || Y == 4;
	case 1: case 3: return Y == 1 || Y == 3;
	default: return false;
	}
}

void CGameState::DoMove (Move m)
{
	m_bGoatsTurn = !m_bGoatsTurn;

	if (m.y1 == -1)
	{
		m_nSquare[m.x2][m.y2] = SQUARE_GOAT;
		m_bUsedGoat[m.x1] = true;
	}
	else
	{
		m_nSquare[m.x2][m.y2] = m_nSquare[m.x1][m.y1];
		m_nSquare[m.x1][m.y1] = SQUARE_EMPTY;
		if (IsTakingMove (m))
		{
			int dx = 0, dy = 0;

			switch (m.x1 - m.x2)
			{
			case 2: dx = -1; break;
			case -2: dx = 1;
			}

			switch (m.y1 - m.y2)
			{
			case 2: dy = -1; break;
			case -2: dy = 1;
			}

			m_nSquare[m.x1 + dx][m.y1 + dy] = SQUARE_EMPTY;
		}
	}
}

int CGameState::GetSquare (int X, int Y)
{
	if (X < 0 || X > 4 || Y < 0 || Y > 4)
		return SQUARE_INVALID;

	return m_nSquare[X][Y];
}

bool CGameState::IsEdgeSquare(int X, int Y)
{
	return X == 0 || X == 4 || Y == 0 || Y == 4;
}

bool CGameState::IsLegalMove (Move m)
{
	// Target square must be empty
	if (m_nSquare[m.TargetSquareX][m.TargetSquareY] != SQUARE_EMPTY)
		return false;

	// If an unused goat is being dropped onto the board
	// it must be goats' turn, the goat must be unused
	// and the target square must be empty
	if (m.SourceSquareY == -1)
	{
		if (!m_bGoatsTurn)
			return false;

		if (m.SourceSquareX < 0 || m.SourceSquareX > 19)
			return false;

		if (m_bUsedGoat[m.SourceSquareX])
			return false;

		if (m.TargetSquareX < 0 || m.TargetSquareX > 4 ||
			m.TargetSquareX < 0 || m.TargetSquareX > 4)
			return false;

		if (m_nSquare[m.TargetSquareX][m.TargetSquareY] != SQUARE_EMPTY)
			return false;

		return true;
	}

	// If a piece is being moved from one square on the board
	// to another then both source and target square must be
	// within the board
	if (m.SourceSquareX < 0 || m.SourceSquareX > 4 ||
		m.SourceSquareY < 0 || m.SourceSquareY > 4 ||
		m.TargetSquareX < 0 || m.TargetSquareX > 4 ||
		m.TargetSquareY < 0 || m.TargetSquareY > 4)
		return false;

	// Pieces may only be moved one square at time, or two
	// if taking
	int dx = m.x1 - m.x2, dy = m.y1 - m.y2;
	switch (dx)
	{
	case -2:
		if (m_nSquare[m.SourceSquareX][m.SourceSquareY] != SQUARE_TIGER)
			return false;
		switch (dy)
		{
		case -2:
			return m_nSquare[m.SourceSquareX+1][m.SourceSquareY+1] == SQUARE_GOAT &&
				CanMoveDownRight (m.SourceSquareX, m.SourceSquareY) &&
				CanMoveDownRight (m.SourceSquareX + 1, m.SourceSquareY + 1);
		case 0:
			return m_nSquare[m.SourceSquareX+1][m.SourceSquareY] == SQUARE_GOAT;

		case 2:
			return m_nSquare[m.SourceSquareX+1][m.SourceSquareY-1] == SQUARE_GOAT &&
				CanMoveUpRight (m.SourceSquareX, m.SourceSquareY) &&
				CanMoveUpRight (m.SourceSquareX + 1, m.SourceSquareY - 1);
		}
		break;

	case -1:
		switch (dy)
		{
		case -1: return CanMoveDownRight (m.SourceSquareX, m.SourceSquareY);
		case 0: return  true;
		case 1: return CanMoveUpRight (m.SourceSquareX, m.SourceSquareY);
		}
		break;

	case 0:
		switch (dy)
		{
		case -2: return m_nSquare[m.SourceSquareX][m.SourceSquareY] == SQUARE_TIGER &&
					 m_nSquare[m.SourceSquareX][m.SourceSquareY+1] == SQUARE_GOAT;
		case -1: case 1: return true;
		case 2: return m_nSquare[m.SourceSquareX][m.SourceSquareY] == SQUARE_TIGER &&
					m_nSquare[m.SourceSquareX][m.SourceSquareY-1] == SQUARE_GOAT;
		}
		break;

	case 1:
		switch (dy)
		{
		case -1: return CanMoveDownLeft (m.SourceSquareX, m.SourceSquareY);
		case 0: return true;
		case 1: return CanMoveUpLeft (m.SourceSquareX, m.SourceSquareY);
		}
		break;

	case 2:
		if (m_nSquare[m.SourceSquareX][m.SourceSquareY] != SQUARE_TIGER)
			return false;
		switch (dy)
		{
		case -2: return m_nSquare[m.SourceSquareX-1][m.SourceSquareY+1] == SQUARE_GOAT &&
					 CanMoveDownLeft (m.SourceSquareX, m.SourceSquareY) &&
					 CanMoveDownLeft (m.SourceSquareX - 1, m.SourceSquareY + 1);
		
		case 0: return m_nSquare[m.SourceSquareX-1][m.SourceSquareY] == SQUARE_GOAT;

		case 2: return m_nSquare[m.SourceSquareX-1][m.SourceSquareY-1] == SQUARE_GOAT &&
					CanMoveUpLeft (m.SourceSquareX, m.SourceSquareY) &&
					CanMoveUpLeft (m.SourceSquareX - 1, m.SourceSquareY - 1);
		}
	}
	
	return false;
}

bool CGameState::IsSquareThreatned (int i, int j)
{
	// Check directly above
	if (GetSquare(i, j - 1) == SQUARE_TIGER &&
		GetSquare(i, j + 1) == SQUARE_EMPTY)
		return true;

	// Check above-left
	if (GetSquare(i - 1, j - 1) == SQUARE_TIGER &&
		GetSquare(i + 1, j + 1) == SQUARE_EMPTY)
		return true;

	// Check to the left
	if (GetSquare(i - 1, j) == SQUARE_TIGER &&
		GetSquare(i + 1, j) == SQUARE_EMPTY)
		return true;

	// Check below-left
	if (GetSquare(i - 1, j + 1) == SQUARE_TIGER &&
		GetSquare(i + 1, j - 1) == SQUARE_EMPTY)
		return true;

	// Check below
	if (GetSquare(i, j + 1) == SQUARE_TIGER &&
		GetSquare(i, j - 1) == SQUARE_EMPTY)
		return true;

	// Check below-right
	if (GetSquare(i + 1, j + 1) == SQUARE_TIGER &&
		GetSquare(i - 1, j - 1) == SQUARE_EMPTY)
		return true;

	// Check to the right
	if (GetSquare(i + 1, j) == SQUARE_TIGER &&
		GetSquare(i - 1, j) == SQUARE_EMPTY)
		return true;
					
	// Check above-right
	if (GetSquare(i + 1, j - 1) == SQUARE_TIGER &&
		GetSquare(i - 1, j + 1) == SQUARE_EMPTY)
		return true;

	return false;
}

bool CGameState::IsTakingMove (Move m)
{
	// This method could do with improving. Currently, it
	// only checks whether the piece is moving two squares

	// Check it isn't an unused goat being dropped onto the board
	if (m.y1 == -1)
		return false;

	int dx = m.SourceSquareX - m.TargetSquareX,
		dy = m.SourceSquareY - m.TargetSquareY;
	return dx == 2 || dx == -2 || dy == 2 || dy == -2;
}

vector <Move> CGameState::ListAllLegalMoves ()
{
	// If it is goats' turn then see if it is legal
	// to drop unused goats onto the board
	if (m_bGoatsTurn)
	{
		for (int i = 0, highest_goat = -1; i < 20; i++)
			if (!m_bUsedGoat[i])
				highest_goat = i;

		if (highest_goat != -1)
		{
			vector <Move> v;
			for (i = 0; i < 5; i++)
				for (int j = 0; j < 5; j++)
					if (m_nSquare[i][j] == SQUARE_EMPTY)
						v.push_back (Move(highest_goat, -1, i, j));

			return v;
		}
	}

	// Otherwise just list all legal moves along the lines on the board

	// Only look at squares that contain a piece that this player can move
	int source_piece = m_bGoatsTurn ? SQUARE_GOAT : SQUARE_TIGER;

	vector <Move> v;
	for (int a = 0; a < 5; a++)
		for (int b = 0; b < 5; b++)
			if (m_nSquare[a][b] == source_piece)
				for (int c = 0; c < 5; c++)
					for (int d = 0; d < 5; d++)
					{
						Move m = Move (a, b, c, d);
						if (IsLegalMove (m))
							v.push_back (m);
					}

	return v;
}

int CGameState::NumGoatsTaken ()
{
	int nGoats = 0;
	for (int i = 0; i < 20; i++)
		if (!m_bUsedGoat[i])
			nGoats++;

	for (i = 0; i < 5; i++)
		for (int j = 0; j < 5; j++)
			if (m_nSquare[i][j] == SQUARE_GOAT)
				nGoats++;

	return 20 - nGoats;
}

int CGameState::NumUnusedGoats()
{
	for (int r = 0, i = 0; i < 20; i++)
		if (!m_bUsedGoat[i])
			r++;

	return r;
}

void CGameState::Reset ()
{
	for (int i = 0; i < 5; i++)
		for (int j = 0; j < 5; j++)
			m_nSquare[i][j] = SQUARE_EMPTY;

	m_nSquare[0][0] = m_nSquare[4][0] = m_nSquare[0][4] =
		m_nSquare[4][4] = SQUARE_TIGER;

	for (i = 0; i < 20; i++)
		m_bUsedGoat[i] = false;
}