/* Copyright (c) Daniel Newman 2006
 *
 * e-mail: danielnewman106@hotmail.com
 */


// GoatAI.cpp: implementation of the CGoatAI class.
//
//////////////////////////////////////////////////////////////////////

#include "GoatAI.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGoatAI::CGoatAI()
{

}

CGoatAI::~CGoatAI()
{

}

Move CGoatAI::MoveToAnyUnthreatenedSquare()
{
	vector <Move> m1 = m_GameState->ListAllLegalMoves(), m2;
	for (int i = 0, s = m1.size(); i < s; i++)
		// Only process goat moves
		if (m_GameState->GetSquare(m1[i].x1, m1[i].y1) == SQUARE_GOAT)
			// Check the square is unthreatened
			if (!m_GameState->IsSquareThreatned(m1[i].TargetSquareX, m1[i].TargetSquareY))
				m2.push_back(m1[i]);

	s = m2.size();
	return s ? m2[rand() % s] : NULL_MOVE;
}

Move CGoatAI::OpeningMove()
{
	int x = rand() % 4;
	if (x == 0)
		return Move (19, -1, 0, 2);
	if (x == 1)
		return Move (19, -1, 2, 0);
	if (x == 2)
		return Move (19, -1, 2, 4);
	return Move (19, -1, 4, 2);
}

Move CGoatAI::PickMove (int Level, CGameState *GameState)
{
	m_GameState = GameState;

	if (Level == 1)
	{
		return RandomMove();
	}

	if (Level == 2)
	{
		/* The irony of this level seems to be that the
		   best strategy seems to be to not threaten
		   the goat pieces, becuase then it will protect
		   them, but to just wait for it to randomly
		   put its pieces into threatened positions	*/

		// Protect any one piece that is being threatened
		Move m = ProtectAnyGoatBeingThreatened(false);
		if (!m.IsNullMove())
			return m;

		// Try to go in any corner
		m = TryToGoInCorner();
		if (!m.IsNullMove())
			return m;

		m = TryToGoOnEdge(false);
		if (!m.IsNullMove())
			return m;

		return RandomMove();
	}

	if (Level == 3)
	{
		/* The only way to beat this level (that I know of)
		   is to "fork" a goat piece so that by using
		   another piece to protect it you would make that
		   piece vulnerable */

		int nUnused = GameState->NumUnusedGoats();
		if (nUnused == 20)
			return OpeningMove();

		// Protect any one piece that is being threatened
		Move m = ProtectAnyGoatBeingThreatened(false);
		if (!m.IsNullMove())
			return m;

		// Try to go in any corner
		m = TryToGoInCorner();
		if (!m.IsNullMove())
			return m;

		m = TryToGoOnEdge(true);
		if (!m.IsNullMove())
			return m;

		m = MoveToAnyUnthreatenedSquare();
		if (!m.IsNullMove())
			return m;

		return RandomMove();
	}

	if (Level >= 4)
	{
		int nUnused = GameState->NumUnusedGoats();
		if (nUnused == 20)
			return OpeningMove();

		// Protect any one piece that is being threatened
		Move m = ProtectAnyGoatBeingThreatened(true);
		if (!m.IsNullMove())
			return m;

		// Try to go in any corner
		m = TryToGoInCorner();
		if (!m.IsNullMove())
			return m;

		m = TryToGoOnEdge(true);
		if (!m.IsNullMove())
			return m;

		m = MoveToAnyUnthreatenedSquare();
		if (!m.IsNullMove())
			return m;

		return RandomMove();
	}

	return RandomMove();
}

Move CGoatAI::ProtectAnyGoatBeingThreatened(bool NewSquareMustBeUnthreatened)
{
	int nUnused = m_GameState->NumUnusedGoats();

	// If there are unused pieces then just drop one
	// onto the board to protect the threatned goat
	if (nUnused > 0)
	{
		for (int i = 0; i < 5; i++)
			for (int j = 0; j < 5; j++)
				if (m_GameState->GetSquare(i, j) == SQUARE_GOAT)
				{
					// Check directly above
					if (m_GameState->GetSquare(i, j - 1) == SQUARE_TIGER &&
						m_GameState->GetSquare(i, j + 1) == SQUARE_EMPTY &&
						(!NewSquareMustBeUnthreatened ||
						!m_GameState->IsSquareThreatned(i, j + 1)))
						return Move (nUnused - 1, -1, i, j + 1);

					// Check above-left
					if (m_GameState->GetSquare(i - 1, j - 1) == SQUARE_TIGER &&
						m_GameState->GetSquare(i + 1, j + 1) == SQUARE_EMPTY &&
						m_GameState->CanMoveDownRight(i - 1, j - 1) &&
						m_GameState->CanMoveDownRight(i, j) &&
						(!NewSquareMustBeUnthreatened ||
						!m_GameState->IsSquareThreatned(i + 1, j + 1)))
						return Move (nUnused - 1, -1, i + 1, j + 1);

					// Check to the left
					if (m_GameState->GetSquare(i - 1, j) == SQUARE_TIGER &&
						m_GameState->GetSquare(i + 1, j) == SQUARE_EMPTY &&
						(!NewSquareMustBeUnthreatened ||
						!m_GameState->IsSquareThreatned(i + 1, j)))
						return Move (nUnused - 1, -1, i + 1, j);

					// Check below-left
					if (m_GameState->GetSquare(i - 1, j + 1) == SQUARE_TIGER &&
						m_GameState->GetSquare(i + 1, j - 1) == SQUARE_EMPTY &&
						m_GameState->CanMoveUpRight(i - 1, j + 1) &&
						m_GameState->CanMoveUpRight(i, j) &&
						(!NewSquareMustBeUnthreatened ||
						!m_GameState->IsSquareThreatned(i + 1, j - 1)))
						return Move (nUnused - 1, -1, i + 1, j - 1);

					// Check below
					if (m_GameState->GetSquare(i, j + 1) == SQUARE_TIGER &&
						m_GameState->GetSquare(i, j - 1) == SQUARE_EMPTY &&
						(!NewSquareMustBeUnthreatened ||
						!m_GameState->IsSquareThreatned(i, j - 1)))
						return Move (nUnused - 1, -1, i, j - 1);

					// Check below-right
					if (m_GameState->GetSquare(i + 1, j + 1) == SQUARE_TIGER &&
						m_GameState->GetSquare(i - 1, j - 1) == SQUARE_EMPTY &&
						m_GameState->CanMoveUpLeft(i + 1, j + 1) &&
						m_GameState->CanMoveUpLeft(i, j) &&
						(!NewSquareMustBeUnthreatened ||
						!m_GameState->IsSquareThreatned(i - 1, j - 1)))
						return Move (nUnused - 1, -1, i - 1, j - 1);

					// Check to the right
					if (m_GameState->GetSquare(i + 1, j) == SQUARE_TIGER &&
						m_GameState->GetSquare(i - 1, j) == SQUARE_EMPTY &&
						(!NewSquareMustBeUnthreatened ||
						!m_GameState->IsSquareThreatned(i - 1, j)))
						return Move (nUnused - 1, -1, i - 1, j);
					
					// Check above-right
					if (m_GameState->GetSquare(i + 1, j - 1) == SQUARE_TIGER &&
						m_GameState->GetSquare(i - 1, j + 1) == SQUARE_EMPTY &&
						m_GameState->CanMoveDownLeft(i + 1, j - 1) &&
						m_GameState->CanMoveDownLeft(i, j) &&
						(!NewSquareMustBeUnthreatened ||
						!m_GameState->IsSquareThreatned(i - 1, j + 1)))
						return Move (nUnused - 1, -1, i - 1, j + 1);
				}
	}
	else
	{

		// Otherwise list all squares that are being threatened and
		// see if there are any moves to save that goat

		// Store the locations of the threatned pieces. SourceSquare
		// is the location of the goat being threatned (i.e. the
		// goat which cannot be moved) and TargetSquare is the empry
		// square
		vector<Move> s;

		for (int i = 0; i < 5; i++)
			for (int j = 0; j < 5; j++)
				if (m_GameState->GetSquare(i, j) == SQUARE_GOAT)
				{
					// Check directly above
					if (m_GameState->GetSquare(i, j - 1) == SQUARE_TIGER &&
						m_GameState->GetSquare(i, j + 1) == SQUARE_EMPTY)
						s.push_back(Move (i, j + 1, i, j));

					// Check above-left
					if (m_GameState->GetSquare(i - 1, j - 1) == SQUARE_TIGER &&
						m_GameState->GetSquare(i + 1, j + 1) == SQUARE_EMPTY)
						s.push_back(Move (i + 1, j + 1, i, j));

					// Check to the left
					if (m_GameState->GetSquare(i - 1, j) == SQUARE_TIGER &&
						m_GameState->GetSquare(i + 1, j) == SQUARE_EMPTY)
						s.push_back(Move (i + 1, j, i, j));

					// Check below-left
					if (m_GameState->GetSquare(i - 1, j + 1) == SQUARE_TIGER &&
						m_GameState->GetSquare(i + 1, j - 1) == SQUARE_EMPTY)
						s.push_back(Move (i + 1, j - 1, i, j));

					// Check below
					if (m_GameState->GetSquare(i, j + 1) == SQUARE_TIGER &&
						m_GameState->GetSquare(i, j - 1) == SQUARE_EMPTY)
						s.push_back(Move (i, j - 1, i, j));

					// Check below-right
					if (m_GameState->GetSquare(i + 1, j + 1) == SQUARE_TIGER &&
						m_GameState->GetSquare(i - 1, j - 1) == SQUARE_EMPTY)
						s.push_back(Move (i - 1, j - 1, i, j));

					// Check to the right
					if (m_GameState->GetSquare(i + 1, j) == SQUARE_TIGER &&
						m_GameState->GetSquare(i - 1, j) == SQUARE_EMPTY)
						s.push_back(Move (i - 1, j, i, j));
					
					// Check above-right
					if (m_GameState->GetSquare(i + 1, j - 1) == SQUARE_TIGER &&
						m_GameState->GetSquare(i - 1, j + 1) == SQUARE_EMPTY)
						s.push_back(Move (i - 1, j + 1, i, j));
				}

		// See if any other goats can move into the empty square
		int size = s.size();
		if (size)
		{
			for (int a = 0; a < size; a++)
				for (i = 0; i < 5; i++)
					for (int j = 0; j < 5; j++)
					{
						int k = s[a].SourceSquareX, l = s[a].SourceSquareY;
						Move a = Move (i, j, k, l);
						if (m_GameState->IsLegalMove(a))
							return a;
					}
		}
	}

	Move a = NULL_MOVE;
	return a;
}

Move CGoatAI::RandomMove()
{
	vector <Move> v = m_GameState->ListAllLegalMoves();
	int s = v.size();
	if (!s)
		return NULL_MOVE;
	return v[rand() % s];
}

Move CGoatAI::TryToGoInCorner()
{
	int nUnused = m_GameState->NumUnusedGoats();
	vector <Move> v;

	// If not all goats have been dropped onto the board
	// and a corner square is available then drop a goat
	// into that square
	if (nUnused > 0)
	{
		vector <Move> v;
		if (m_GameState->GetSquare(0, 0) == SQUARE_EMPTY)
			v.push_back(Move (nUnused - 1, - 1, 0, 0));
		if (m_GameState->GetSquare(4, 0) == SQUARE_EMPTY)
			v.push_back(Move (nUnused - 1, - 1, 4, 0));
		if (m_GameState->GetSquare(0, 4) == SQUARE_EMPTY)
			v.push_back(Move (nUnused - 1, - 1, 0, 4));
		if (m_GameState->GetSquare(4, 4) == SQUARE_EMPTY)
			v.push_back(Move (nUnused - 1, - 1, 4, 4));
		int s = v.size();
		if (s)
			return v[rand() % s];
		else
			return NULL_MOVE;
	}

	// Otherwise see if there are any goat pieces
	// that can be moved into a corner square
	for (int i = 0; i < 5; i++)
		for (int j = 0; j < 5; j++)
		{
			Move m = Move(i, j, 0, 0);
			if (m_GameState->GetSquare(i, j) == SQUARE_GOAT &&
				m_GameState->IsLegalMove(m))
				v.push_back(m);

			m = Move(i, j, 4, 0);
			if (m_GameState->GetSquare(i, j) == SQUARE_GOAT &&
				m_GameState->IsLegalMove(m))
				v.push_back(m);

			m = Move(i, j, 0, 4);
			if (m_GameState->GetSquare(i, j) == SQUARE_GOAT &&
				m_GameState->IsLegalMove(m))
				v.push_back(m);

			m = Move(i, j, 4, 4);
			if (m_GameState->GetSquare(i, j) == SQUARE_GOAT &&
				m_GameState->IsLegalMove(m))
				v.push_back(m);
		}

	return NULL_MOVE;
}

Move CGoatAI::TryToGoOnEdge(bool bMustBeUnthreatened)
{
	// If there are unused goats then drop on onto
	// any square on the edge of the board
	int nUnused = m_GameState->NumUnusedGoats();
	if (nUnused > 0)
	{
		vector<Move> m;	// List of all empty edge-squares
		for (int i = 0; i < 5; i++)
		{
			if (m_GameState->GetSquare(i, 0) == SQUARE_EMPTY)
				if (!bMustBeUnthreatened || !m_GameState->IsSquareThreatned(i, 0))
					m.push_back(Move(nUnused - 1, -1, i, 0));
			if (m_GameState->GetSquare(0, i) == SQUARE_EMPTY)
				if (!bMustBeUnthreatened || !m_GameState->IsSquareThreatned(0, i))
					m.push_back(Move(nUnused - 1, -1, 0, i));
			if (m_GameState->GetSquare(i, 4) == SQUARE_EMPTY)
				if (!bMustBeUnthreatened || !m_GameState->IsSquareThreatned(i, 4))
					m.push_back(Move(nUnused - 1, -1, i, 4));
			if (m_GameState->GetSquare(4, i) == SQUARE_EMPTY)
				if (!bMustBeUnthreatened || !m_GameState->IsSquareThreatned(4, i))
					m.push_back(Move(nUnused - 1, -1, 4, i));
		}
		int s = m.size();
		return s ? m[rand() % s] : NULL_MOVE;
	}

	// Otherwise see if any goat pieces can be moved to
	// edge squares
	vector <Move> m1 = m_GameState->ListAllLegalMoves(), m2;
	for (int i = 0, s = m1.size(); i < s; i++)
		if (m_GameState->IsEdgeSquare(m1[i].TargetSquareX, m1[i].TargetSquareY))
			if (!bMustBeUnthreatened ||
				!m_GameState->IsSquareThreatned(m1[i].TargetSquareX, m1[i].TargetSquareY))
				m2.push_back(m1[i]);
	s = m2.size();
	return s ? m2[rand() % s] : NULL_MOVE;
}