/* Copyright (c) Daniel Newman 2006
 *
 * e-mail: danielnewman106@hotmail.com
 */



#define _WIN32_WINNT 0x0500
#define STRSAFE_NO_DEPRECATE
#include <windows.h>
#include "ddutil.h"
#include "MainWindow.h"
#include "BackgroundSurface.h"
#include "GameState.h"
#include "GoatAI.h"
#include "MainMenu.h"
#include "MP3File.h"
#include "resource.h"

#define MUSIC_TITLESCREEN (101)
#define MUSIC_CREDITS (102)

CMainWindow *Wnd;
CDisplay *Display = new CDisplay();
CMainMenu *MainMenu;
CBackgroundSurface *Background;
CGameState *GameState = NULL;
CGoatAI *GoatAI = new CGoatAI();
CMP3File *Music = NULL;

int GoatPlayer, TigerPlayer;
enum
{
	PLAYER_HUMAN,
	PLAYER_COMPUTER,
	PLAYER_NETWORK
};
int nLevel;
POINT SelectedPiece, MousePos, DraggedPieceOffset;
float fAnimationPercentage = 1.5f;
#define ANIMATIONDELAY_FAST 15
#define ANIMATIONDELAY_NORMAL 20
#define ANIMATIONDELAY_SLOW 40
int nAnimatedMoveDelay = ANIMATIONDELAY_NORMAL;
Move AnimatedMove = NULL_MOVE;
int nCreditsFrame = -1;
int nTimeCountDown = 0, nTimeLimit = -1;

bool bMainMenuVisible = true;
bool bBoardVisible = false;
bool bSound = true, bMusic = true;
int nPiecesTigersMustEat = 5;

HDC hdcGoat[7], hdcTiger[7];

LPSTR szCredit[] = {
	"Programming", "DANIEL NEWMAN",
	"Artwork", "BEN GASKELL",
	"Music", "JAMIE GRAY",
	"Test Driving", "PAUL BROADBENT",
	"Gaffer", "BOB SRISKANTHARAJAH",
	"Icons courtesy of", "www.flamingtext.com",
	"Sounds courtesy of", "www.a1freesoundeffects.com",
	"GOATS AND TIGERS", "goatsandtigers.allhere.com"
};


void AI_Goat();
void AI_Tiger();
void DrawGoat(HDC hdc, int X, int Y, int nGoat);
void DrawTiger(HDC hdc, int X, int Y, int nTiger);
POINT GetSquareFromClick(int X, int Y);
POINT GetSquarePos(int X, int Y);
void GoToCredits();
void GoToGameOverScreen();
void GoToMainMenu(bool bRestartMusic);
void GoToRules();
void GoToVictoryScreen();
void LoadGoatBMP (int Level);
void LoadTigerBMP (int Level);
void NewSinglePlayerGame (bool bHumanGoatPlayer);
void OnGraphEvent();
void (*OnKeyDown)(int Key);
void OnKeyDown_Board(int Key);
void OnKeyDown_DoNothing(int Key) { return; }
void OnKeyDown_MainMenu(int Key);
void OnKeyDown_Rules(int Key) { GoToMainMenu(false); }
void (*OnLeftButtonDown)(int X, int Y);
void (*OnLeftButtonUp)(int X, int Y);
void OnLeftButtonDown_Board (int X, int Y);
void OnLeftButtonUp_Board (int X, int Y);
void OnLeftButtonDown_DoNothing (int, int) { return; }
void OnLeftButtonDown_GoToMainMenu (int, int);
void OnLeftButtonDown_MainMenu(int X, int Y);
void OnLeftButtonDown_NextLevel (int, int);
void OnLeftButtonDown_Rules(int, int) { GoToMainMenu(false); }
void OnMouseMove(int X, int Y);
void OnPaint ();
void OnTimer_AnimatedMove ();
void OnTimer_Credits ();
void OnTimer_TimeAttack();
void PlayMusic(int level);
void RandomScream ();
void ResetBoard ();
void StartAnimatedMove (Move m);
void StopMusic();
Move TigerAI ();

#define TIMER_ANIMATEDMOVE (WM_USER+1)
#define TIMER_CREDITS (WM_USER+2)
#define TIMER_GAMEOVER (WM_USER+3)
#define TIMER_TIMEATTACK (WM_USER+4)
#define CREDIT_DELAY 20

inline bool IsAnyPieceSelected ()
{
	return SelectedPiece.x != -1 || SelectedPiece.y != -1;
}

inline void UnselectAllPieces ()
{
	SelectedPiece.x = SelectedPiece.y = -1;
}

LRESULT CALLBACK WndProc (HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch (Msg)
	{
	case WM_TIMER:
		switch (wParam)
		{
		case TIMER_ANIMATEDMOVE:
			OnTimer_AnimatedMove();
			break;

		case TIMER_GAMEOVER:
			KillTimer (hWnd, TIMER_GAMEOVER);
			GoToGameOverScreen();
			break;

		case TIMER_CREDITS:
			OnTimer_Credits ();
			break;

		case TIMER_TIMEATTACK:
			OnTimer_TimeAttack();
			break;
		}

		break;

	case WM_MP3FILEEVENT:
		OnGraphEvent();
		break;

	case WM_LBUTTONDOWN:
		OnLeftButtonDown(LOWORD(lParam), HIWORD(lParam));
		OnMouseMove(LOWORD(lParam), HIWORD(lParam));
		break;

	case WM_LBUTTONUP:
		OnLeftButtonUp (LOWORD(lParam), HIWORD(lParam));
		break;

	case WM_MOUSEMOVE:
		OnMouseMove(LOWORD(lParam), HIWORD(lParam));
		break;

	case WM_KEYDOWN:
		OnKeyDown((int)wParam);
		break;

	case WM_PAINT:
		ValidateRect (Wnd->GetHWnd (), NULL);
		OnPaint ();
		break;

	case WM_MOVE:
		Display->UpdateBounds ();
		break;

	case WM_DESTROY:
		PostQuitMessage (0);
		break;

	default:
		return DefWindowProc (hWnd, Msg, wParam, lParam);
	}

	return 0L;
}

void AI_Goat ()
{
	vector <Move> v = GameState->ListAllLegalMoves ();
	int s = v.size();

	// If more than the max number of goats have been
	// taken or there are no legal moves then surrender
	if (GameState->NumGoatsTaken() >= nPiecesTigersMustEat || !s)
	{
		GoToVictoryScreen ();
		return;
	}

	Move m = GoatAI->PickMove(nLevel, GameState);

	// Do not show animation if in "Time Attack" mode
	if (nTimeLimit > 0)
	{
		GameState->DoMove(m);
		OnPaint();
	}
	else
		StartAnimatedMove (m);
}

void AI_Tiger ()
{
	vector <Move> v = GameState->ListAllLegalMoves ();
	int s = v.size();
	
	// If there are no legal moves then surrender
	if (!s)
	{
		GoToVictoryScreen ();
		return;
	}

	// Pick a move
	Move m = NULL_MOVE;

		// See if it possible to take
	for (int i = 0; i < s; i++)
		if (GameState->IsTakingMove (v[i]))
		{
			m = v[i];
			break;
		}

		// Otherwise pick a random move
	if (m == NULL_MOVE)
		m = v[rand() % s];

	if (nTimeLimit > 0)
	{
		GameState->DoMove(m);
		if (GameState->IsTakingMove(m))
			RandomScream();
		OnPaint();
	}
	else
		StartAnimatedMove (m);
}

void DrawGoat (HDC hdc, int X, int Y, int nGoat)
{
	POINT p = GetSquarePos (X, Y);
	TransparentBlt (hdc, p.x - 16, p.y - 16, 32, 32, hdcGoat[nGoat], 0, 0, 32, 32, 0xFF00FF);
}

void DrawTiger (HDC hdc, int X, int Y, int nTiger)
{
	POINT p = GetSquarePos (X, Y);
	TransparentBlt (hdc, p.x - 16, p.y - 16, 32, 32, hdcTiger[nTiger], 0, 0, 32, 32, 0xFF00FF);
}

POINT GetSquareFromClick (int X, int Y)
{
	// See if a square on the board was clicked on
	for (int i = 0; i < 5; i++)
		for (int j = 0; j < 5; j++)
		{
			POINT p = GetSquarePos (i, j);
			int dx = p.x - X, dy = p.y - Y;
			if (dx > -16 && dx < 16 && dy > -16 && dy < 16)
			{
				p.x = i;
				p.y = j;
				return p;
			}
		}

	// See if an unused goat was clicked on
	for (i = 0; i < 20; i++)
	{
		POINT p = GetSquarePos (i, -1);
		int dx = p.x - X, dy = p.y - Y;
		if (dx > -16 && dx < 16 && dy > -16 && dy < 16)
		{
			p.x = i;
			p.y = -1;
			return p;
		}
	}

	// Otherwise assume nothing was clicked on
	POINT p;
	p.x = p.y = -1;
	return p;
}

POINT GetSquarePos (int X, int Y)
{

	POINT p;
	p.x = p.y = -50;
	if (Y == -1)
	{
		p.y = 85 + ((X % 10) * 50);
		p.x = X < 10 ? 80 : 732;
	}
	else
	{
		switch (X)
		{
		case 0: p.x = 171; break;
		case 1: p.x = 287; break;
		case 2: p.x = 401; break;
		case 3: p.x = 517; break;
		case 4: p.x = 631; break;
		}

		if (X != -1)
			switch (Y)
			{
			case 0: p.y = 95; break;
			case 1: p.y = 210; break;
			case 2: p.y = 326; break;
			case 3: p.y = 440; break;
			case 4: p.y = 556; break;
			}
	}

	return p;
}

void GoToBoard ()
{
	AnimatedMove.x1 = -1;
	bMainMenuVisible = false;
	MainMenu->ReturnToTop();
	Background->LoadBackground (BACKGROUND_BOARD);
	bBoardVisible = true;
	OnKeyDown = OnKeyDown_Board;
	OnLeftButtonDown = OnLeftButtonDown_Board;
	OnLeftButtonUp = OnLeftButtonUp_Board;
	OnPaint ();

	// If in "Time Attack" mode then restart the timer
	if (nTimeLimit)
		SetTimer(Wnd->GetHWnd(), TIMER_TIMEATTACK, 1000, NULL);
}

void GoToCredits ()
{
	bMainMenuVisible = false;
	if (GameState)
	{
		delete GameState;
		GameState = NULL;
	}
	Background->LoadBackground (BACKGROUND_CREDITS);
	bBoardVisible = false;
	OnLeftButtonDown = OnLeftButtonDown_DoNothing;
	OnLeftButtonDown = OnLeftButtonDown_GoToMainMenu;

	// Stop any sounds
	if (bSound)
		PlaySound (NULL, NULL, 0);

	nCreditsFrame = 0;

	OnPaint ();
	PlayMusic(MUSIC_CREDITS);

	SetTimer (Wnd->GetHWnd(), TIMER_CREDITS, CREDIT_DELAY, NULL);
}

void GoToGameOverScreen ()
{
	UnselectAllPieces();
	MainMenu->SetCanResume (false);
	bMainMenuVisible = false;
	StopMusic();
	if (GameState)
	{
		delete GameState;
		GameState = NULL;
	}
	Background->LoadBackground (BACKGROUND_GAMEOVER);
	bBoardVisible = false;
	OnLeftButtonDown = OnLeftButtonDown_DoNothing;
	OnLeftButtonUp = OnLeftButtonDown_GoToMainMenu;
	if (bSound)
		PlaySound (".\\Sounds\\gameover.wav", NULL, SND_ASYNC | SND_FILENAME | SND_NODEFAULT);
	OnPaint ();
}

void GoToMainMenu (bool bRestartMusic)
{
	UnselectAllPieces();
	KillTimer(Wnd->GetHWnd(), TIMER_TIMEATTACK);
	if (bRestartMusic || GameState == NULL)
		PlayMusic(MUSIC_TITLESCREEN);
	Background->LoadBackground (BACKGROUND_TITLESCREEN);
	bBoardVisible = false;
	bMainMenuVisible = true;
	nCreditsFrame = -1;
	OnKeyDown = OnKeyDown_MainMenu;
	OnLeftButtonDown = OnLeftButtonDown_MainMenu;
	OnLeftButtonUp = OnLeftButtonDown_DoNothing;
	OnPaint ();
}

void GoToRules ()
{
	OnKeyDown = OnKeyDown_Rules;
	OnLeftButtonDown = OnLeftButtonDown_Rules;
	OnLeftButtonUp = OnLeftButtonDown_DoNothing;
	bMainMenuVisible = false;
	Background->LoadBackground (BACKGROUND_RULES);
	bBoardVisible = false;
	OnPaint ();
}

void GoToVictoryScreen ()
{
	KillTimer(Wnd->GetHWnd(), TIMER_TIMEATTACK);
	bMainMenuVisible = false;
	StopMusic();
	if (GameState)
	{
		delete GameState;
		GameState = NULL;
	}

	Background->LoadBackground (BACKGROUND_VICTORY);
	bBoardVisible = false;
	if (bSound)
		PlaySound (".\\Sounds\\Victory.wav", NULL, SND_ASYNC | SND_FILENAME | SND_NODEFAULT);
	OnLeftButtonDown = OnLeftButtonDown_DoNothing;
	OnLeftButtonUp = OnLeftButtonDown_NextLevel;
	OnPaint ();	
}

void LoadGoatAndTigerBMPs ()
{
	// Get the DirectDrawSurface DC
	HDC hdc;
	if (FAILED (Display->GetBackBuffer()->GetDC (&hdc)))
		return;

	// Create Compatible DC's
	for (int i = 0; i < 7; i++)
	{
		hdcGoat[i] = CreateCompatibleDC (hdc);
		hdcTiger[i] = CreateCompatibleDC (hdc);
	}
	Display->GetBackBuffer()->ReleaseDC (hdc);

	// Load the BMP from the DLL
	HMODULE hDLL = LoadLibrary ("Backgrounds.dll");
	if (!hDLL)
		return;
	for (i = 0; i < 7; i++)
	{
		SelectObject (hdcGoat[i], LoadImage (hDLL, MAKEINTRESOURCE(115+i), IMAGE_BITMAP, 0, 0, 0));
		SelectObject (hdcTiger[i], LoadImage (hDLL, MAKEINTRESOURCE(102+i), IMAGE_BITMAP, 0, 0, 0));
	}
	FreeLibrary (hDLL);
}

void NewSinglePlayerGame (bool bHumanGoatPlayer)
{
	// Stop all timers
	KillTimer(Wnd->GetHWnd(), TIMER_TIMEATTACK);
	KillTimer (Wnd->GetHWnd(), TIMER_CREDITS);
	KillTimer (Wnd->GetHWnd(), TIMER_ANIMATEDMOVE);

	// Allow te user to continue this game if they return to the main menu
	MainMenu->SetCanResume (true);

	// Store whether the player is goats or tigers
	if (bHumanGoatPlayer)
	{
		GoatPlayer = PLAYER_HUMAN;
		TigerPlayer = PLAYER_COMPUTER;
	}
	else
	{
		GoatPlayer = PLAYER_COMPUTER;
		TigerPlayer = PLAYER_HUMAN;
	}

	nLevel = 1;

	// If in "Time Attack" mode then don't start music
	// until countdown has finished
	if (!nTimeCountDown)
		PlayMusic(1);
	else
		Music->Stop();

	GoToBoard ();
	ResetBoard ();

	if (GoatPlayer == PLAYER_COMPUTER)
		AI_Goat ();

	OnPaint ();
}

void OnGraphEvent()
{
	if (!Music)
		return;

	long code;
	__int64 param1, param2;
	while (Music->GetMediaEvent()->GetEvent(&code, &param1, &param2, 0) != E_ABORT)
	{
		if (code == EC_COMPLETE)
		{
			Music->Stop();
			Music->Rewind();
			Music->Play();
		}
	}
}

void OnKeyDown_Board(int Key)
{
	switch (Key)
	{
	case VK_ESCAPE:
		Music->Stop();
		GoToMainMenu(false);
		break;
	}	
}

void OnKeyDown_MainMenu(int Key)
{
	switch (Key)
	{
	case VK_ESCAPE:
		if (MainMenu->CanResume())
			GoToBoard();
		break;
	}
}

void OnLeftButtonDown_Board (int X, int Y)
{
	// If the board is visible then see if a piece was clicked on
	if (bBoardVisible)
	{
		UnselectAllPieces ();	

		// See if a square on the board was clicked on
		for (int i = 0; i < 5; i++)
			for (int j = 0; j < 5; j++)
			{
				if ((GameState->GetSquare(i, j) == SQUARE_GOAT && GoatPlayer == PLAYER_HUMAN &&
					!GameState->NumUnusedGoats() && GameState->IsGoatsTurn()) ||
					(GameState->GetSquare(i, j) == SQUARE_TIGER && TigerPlayer == PLAYER_HUMAN &&
					!GameState->IsGoatsTurn()))
				{
					POINT p = GetSquarePos (i, j);
					int dx = p.x - X, dy = p.y - Y;
					if (dx > -16 && dx < 16 && dy > -16 && dy < 16)
					{
						SelectedPiece.x = i;
						SelectedPiece.y = j;
						DraggedPieceOffset.x = 16 - dx;
						DraggedPieceOffset.y = 16 - dy;
					}
				}
			}

		// See if an unused goat was clicked on
		if (GoatPlayer == PLAYER_HUMAN)
			for (i = 0; i < 20; i++)
				if (!GameState->IsGoatUsed(i))
				{
					POINT p = GetSquarePos (i, -1);
					int dx = p.x - X, dy = p.y - Y;
					if (dx > -16 && dx < 16 && dy > -16 && dy < 16)
					{
						SelectedPiece.x = i;
						SelectedPiece.y = -1;
						DraggedPieceOffset.x = 16 - dx;
						DraggedPieceOffset.y = 16 - dy;
					}
				}
	}
}

void OnLeftButtonDown_GoToMainMenu (int, int)
{
	KillTimer (Wnd->GetHWnd(), TIMER_CREDITS);
	nCreditsFrame = -1;
	GoToMainMenu(false);
}

void OnLeftButtonDown_MainMenu(int X, int Y)
{
	LPSTR s = MainMenu->OnMouseDown(X, Y);

	if (!strcmp(s, "RESUME"))
	{
		GoToBoard();
		Music->Play();
		return;
	}

	if (!strcmp(s, "PLAY AS GOATS"))
	{
		KillTimer(Wnd->GetHWnd(), TIMER_TIMEATTACK);
		nTimeLimit = nTimeCountDown = 0;
		NewSinglePlayerGame(true);
		return;
	}

	if (!strcmp(s, "PLAY AS TIGERS"))
	{
		KillTimer(Wnd->GetHWnd(), TIMER_TIMEATTACK);
		nTimeLimit = nTimeCountDown = 0;
		NewSinglePlayerGame(false);
		return;
	}

	if (!strcmp(s, "TIME ATTACK"))
	{
		nTimeLimit = 60;
		nTimeCountDown = 3;
		SetTimer(Wnd->GetHWnd(), TIMER_TIMEATTACK, 1000, NULL);
		NewSinglePlayerGame(true);
		return;
	}

	if (!strcmp(s, "RULES"))
	{
		GoToRules();
		return;
	}
	
	if (!strcmp(s, "EXIT"))
	{
		PostQuitMessage(0);
		return;
	}

	if (!strcmp(s, "SOUND NO"))
	{
		bSound = false;
		OnPaint();
		return;
	}

	if (!strcmp(s, "SOUND YES"))
	{
		bSound = true;
		OnPaint();
		return;
	}

	if (!strcmp(s, "MUSIC NO"))
	{
		bMusic = false;
		Music->SetVolume(bMusic ? 0 : -9000);
		OnPaint();
		return;
	}

	if (!strcmp(s, "MUSIC YES"))
	{
		bMusic = true;
		Music->SetVolume(bMusic ? 0 : -9000);
		OnPaint();
		return;
	}
	
	if (!strcmp(s, "ANIMATION FAST"))
	{
		nAnimatedMoveDelay = ANIMATIONDELAY_FAST;
		OnPaint();
		return;
	}

	if (!strcmp(s, "ANIMATION NORMAL"))
	{
		nAnimatedMoveDelay = ANIMATIONDELAY_NORMAL;
		OnPaint();
		return;
	}

	if (!strcmp(s, "ANIMATION SLOW"))
	{
		nAnimatedMoveDelay = ANIMATIONDELAY_SLOW;
		OnPaint();
		return;
	}

	if (!strcmp(s, "TIGERS MUST EAT 1 GOAT"))
	{
		nPiecesTigersMustEat = 1;
		OnPaint();
		return;
	}
	
	if (!strcmp(s, "TIGERS MUST EAT 3 GOATS"))
	{
		nPiecesTigersMustEat = 3;
		OnPaint();
		return;
	}

	if (!strcmp(s, "TIGERS MUST EAT 5 GOATS"))
	{
		nPiecesTigersMustEat = 5;
		OnPaint();
		return;
	}

	if (!strcmp(s, "TIGERS MUST EAT 7 GOATS"))
	{
		nPiecesTigersMustEat = 7;
		OnPaint();
		return;
	}

	if (!strcmp(s, "TIGERS MUST EAT ALL GOATS"))
	{
		nPiecesTigersMustEat = 20;
		OnPaint();
		return;
	}
/*	// If anything other than an empty string is
	// returned then we should to repaint
	if (strcmp(s, ""))
		OnPaint();*/
}

void OnLeftButtonDown_NextLevel (int, int)
{
	nLevel++;
	if (nLevel > 7)
	{
		GoToCredits ();
		return;
	}

	// If in "Time Attack" mode then set the timer
	if (nTimeLimit)
	{
		nTimeCountDown = 3;
		switch (nLevel)
		{
		case 2: nTimeLimit = 50; break;
		case 3: nTimeLimit = 40; break;
		case 4: nTimeLimit = 30; break;
		case 5: nTimeLimit = 20; break;
		case 6: nTimeLimit = 15; break;
		case 7: default: nTimeLimit = 10;
		}
	}

	// Stop the "clapping" sound and play the music for the next level
	PlaySound(NULL, NULL, 0);

	// If in "Time Attack" mode then don't start
	// the music until the count down finishes
	if (!nTimeCountDown)
		PlayMusic(nLevel);
	else
		Music->Stop();


	ResetBoard();

	if (GoatPlayer == PLAYER_COMPUTER)
		AI_Goat ();

	GoToBoard ();
}

void OnLeftButtonUp_Board (int X, int Y)
{
	// If in "Time Attack" mode then do not allow
	// the user to move before the time starts
	if (nTimeCountDown)
	{
		UnselectAllPieces();
		OnPaint();
		return;
	}

	// See if a piece was dropped on the board
	if (bBoardVisible && IsAnyPieceSelected())
	{
		POINT p = GetSquareFromClick(X, Y);
		Move m = Move(SelectedPiece.x, SelectedPiece.y, p.x, p.y);
		UnselectAllPieces();
		if (p.x == -1)
		{
			OnPaint ();
			return;
		}
		if (GameState->IsLegalMove(m))
		{
			GameState->DoMove(m);

			if (GameState->IsTakingMove(m))
				RandomScream();

			OnPaint();
			if (GameState->IsGoatsTurn())
			{
				if (GoatPlayer == PLAYER_COMPUTER)
					AI_Goat();
			}
			else
			{
				if (TigerPlayer == PLAYER_COMPUTER)
					AI_Tiger();
			}
			return;
		}
		OnPaint();
	}
}

void OnMouseMove(int X, int Y)
{
	if (bBoardVisible && IsAnyPieceSelected())
	{
		MousePos.x = X;
		MousePos.y = Y;
		OnPaint ();
		return;
	}
	if (bMainMenuVisible)
	{
		MainMenu->OnMouseMove(X, Y);
		if (MainMenu->MustRepaint())
			OnPaint();
		return;
	}
}

void OnPaint ()
{
	Background->DrawBackground ();

	// Draw the Main Menu
	if (bMainMenuVisible)
	{
		MainMenu->Draw(Display->GetBackBuffer());
		Display->Present();
		return;
	}

	// Draw the board
	if (bBoardVisible && GameState)
	{
		int nGoat = GoatPlayer == PLAYER_HUMAN ? 0 : nLevel - 1,
			nTiger = TigerPlayer == PLAYER_HUMAN ? 0 : nLevel - 1;
		HDC hdc;
		if (FAILED (Display->GetBackBuffer()->GetDC (&hdc)))
			return;
		for (int i = 0; i < 5; i++)
			for (int j = 0; j < 5; j++)
				// Don't draw the piece if it is being animated or it
				// is being taken and the taking goat has already
				// passed over it
				if (AnimatedMove.x1 != i || AnimatedMove.y1 != j)
					if (i != (AnimatedMove.x1 + AnimatedMove.x2) / 2 ||
						j != (AnimatedMove.y1 + AnimatedMove.y2) / 2 ||
						fAnimationPercentage < 0.5f)
						switch (GameState->GetSquare (i, j))
						{
						case SQUARE_GOAT:
							if (SelectedPiece.x != i || SelectedPiece.y != j)
								DrawGoat (hdc, i, j, nGoat);
							break;

						case SQUARE_TIGER:
							if (SelectedPiece.x != i || SelectedPiece.y != j)
								DrawTiger (hdc, i, j, nTiger);
							break;
						}
		for (i = 0; i < 20; i++)
			if (!GameState->IsGoatUsed (i))
			{
				// Don't draw the piece if it is being animated or dragged
				if (SelectedPiece.x != i || SelectedPiece.y != -1)
					if (AnimatedMove.x1 != i || AnimatedMove.y1 != -1)
						DrawGoat (hdc, i, -1, nGoat);
			}

		// Draw the animated move (if there is one)
		if (fAnimationPercentage <= 1.0f)
		{
			POINT p1 = GetSquarePos (AnimatedMove.x1, AnimatedMove.y1),
				p2 = GetSquarePos (AnimatedMove.x2, AnimatedMove.y2);
			int dx = p2.x - p1.x, dy = p2.y - p1.y;

			TransparentBlt (hdc, p1.x + (dx * fAnimationPercentage) - 16,
				p1.y + (dy * fAnimationPercentage) - 16, 32, 32, 
				GameState->IsGoatsTurn() ? hdcGoat[nGoat] : hdcTiger[nTiger],
				0, 0, 32, 32, 0xFF00FF);
		}


		// Draw the time limit (if there is one)
		if (nTimeCountDown)
		{
			char t[3];
			wsprintf(t, "%d", nTimeCountDown);
			RECT r = { 330, 230, 470, 370 };
			FillRect (hdc, &r, (HBRUSH)GetStockObject(WHITE_BRUSH));
			HFONT hFont = CreateFont(-MulDiv(72, GetDeviceCaps(hdc, LOGPIXELSY), 72),
				0, 0, 0, FW_BOLD, FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS,
				CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, "Arial");
			HGDIOBJ hOldFont = SelectObject(hdc, hFont);
			DrawText(hdc, t, strlen(t), &r, DT_CENTER | DT_VCENTER | DT_NOCLIP | DT_SINGLELINE);
			SelectObject(hdc, hOldFont);
			DeleteObject(hFont);
			
		}
		else if (nTimeLimit > 0)
		{
			char t[4];
			wsprintf (t, "%d%s", nTimeLimit, nTimeLimit < 10 ? " " : "");
			RECT r = { 5, 5, 30, 25 };
			FillRect(hdc, &r, (HBRUSH)GetStockObject(WHITE_BRUSH));
			SetBkColor(hdc, 0xFFFFFF);
			HFONT hFont = CreateFont(-MulDiv(12, GetDeviceCaps(hdc, LOGPIXELSY), 72),
				0, 0, 0, FW_BOLD, FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS,
				CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, "Arial");
			HGDIOBJ hOldFont = SelectObject(hdc, hFont);
			DrawText(hdc, t, strlen(t), &r, DT_CENTER | DT_SINGLELINE | DT_VCENTER | DT_NOCLIP);
			SelectObject(hdc, hOldFont);
			DeleteObject(hFont);
			//TextOut(hdc, 5, 3, t, strlen(t));
			HGDIOBJ hOldPen = SelectObject(hdc, CreatePen(PS_SOLID, 1, 0)),
				hOldBrush = SelectObject(hdc, CreateSolidBrush(0xFF));
			RoundRect(hdc, 32, 4, 32 + (nTimeLimit * 2), 11, 3, 3);
			DeleteObject(SelectObject(hdc, hOldPen));
			DeleteObject(SelectObject(hdc, hOldBrush));
		}

		// Draw the dragged piece
		if (IsAnyPieceSelected())
		{
			TransparentBlt (hdc, MousePos.x - DraggedPieceOffset.x,
				MousePos.y - DraggedPieceOffset.y, 32, 32, 
				GoatPlayer == PLAYER_HUMAN ? hdcGoat[nGoat] : hdcTiger[nTiger],
				0, 0, 32, 32, 0xFF00FF);
		}
		Display->GetBackBuffer()->ReleaseDC (hdc);
	}

	// Draw the credits
	if (nCreditsFrame != -1)
	{
		HDC hdc;
		if (FAILED (Display->GetBackBuffer()->GetDC (&hdc)))
			return;

#define CLR_PINK 0xFF00FF
		HDC hdcOffscreen = CreateCompatibleDC (hdc);
		HGDIOBJ hOldBMP = SelectObject (hdcOffscreen, CreateCompatibleBitmap (hdc, 500, 100));
		SelectObject (hdcOffscreen, GetStockObject (DC_BRUSH));
		SetDCBrushColor (hdcOffscreen, CLR_PINK);
		SelectObject (hdcOffscreen, GetStockObject (DC_PEN));
		SetDCPenColor (hdcOffscreen, CLR_PINK);
		SetBkColor (hdcOffscreen, CLR_PINK);
		SetTextColor (hdcOffscreen, 0xFFFFFF);
		RECT r = { 0, 0, 500, 100 };
		HGDIOBJ hOldFont = SelectObject (hdcOffscreen,
			CreateFont (-MulDiv(24, GetDeviceCaps(hdcOffscreen, LOGPIXELSY), 72), 0, 0, 0,
			FW_EXTRABOLD, FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_DEFAULT_PRECIS,
			CLIP_DEFAULT_PRECIS, DRAFT_QUALITY, DEFAULT_PITCH, "Arial"));

		for (int i = 0; i < 15; i += 2)
		{
			// Erase the previous text
			Rectangle (hdcOffscreen, 0, 0, 500, 100);

			// Draw text onto an offscreen text
			DrawText (hdcOffscreen, szCredit[i], strlen(szCredit[i]), &r,
				DT_SINGLELINE | DT_CENTER | DT_TOP | DT_NOCLIP);
			// Copy the text but not the background
			TransparentBlt (hdc, 150, 600 - nCreditsFrame + (100 * i), 500, 100,
				hdcOffscreen, 0, 0, 500, 100, 0xFF00FF);

			DrawText (hdcOffscreen, szCredit[i+1], strlen(szCredit[i+1]), &r,
				DT_SINGLELINE | DT_CENTER | DT_TOP | DT_NOCLIP);
			// Copy the text but not the background
			TransparentBlt (hdc, 150, 650 - nCreditsFrame + (100 * i), 500, 100,
				hdcOffscreen, 0, 0, 500, 100, 0xFF00FF);

		}
		DeleteObject (SelectObject (hdcOffscreen, hOldBMP));
		DeleteObject (SelectObject (hdcOffscreen, hOldFont));
		DeleteDC (hdcOffscreen);

		Display->GetBackBuffer()->ReleaseDC (hdc);
	}

	Display->Present ();
}

void OnTimer_AnimatedMove ()
{
	float before = fAnimationPercentage;
	fAnimationPercentage += 0.02f;

	// Scream when "fAnimationPercentage" passes 0.5
	if (fAnimationPercentage >= 0.5f && before < 0.5f && GameState->IsTakingMove(AnimatedMove))
		RandomScream ();

	if (fAnimationPercentage < 1.0f)
		SetTimer (Wnd->GetHWnd(), TIMER_ANIMATEDMOVE, nAnimatedMoveDelay, NULL);
	else
	{
		KillTimer (Wnd->GetHWnd(), TIMER_ANIMATEDMOVE);
		GameState->DoMove (AnimatedMove);

		// If there are no legal moves or the maximum number of goats
		// have been eaten then it's game over
		if ((GoatPlayer == PLAYER_HUMAN && GameState->NumGoatsTaken() >= nPiecesTigersMustEat) ||			
			!GameState->ListAllLegalMoves().size())
		{
			SetTimer (Wnd->GetHWnd(), TIMER_GAMEOVER, 1000, NULL);
			return;
		}
	}

	OnPaint ();
}

void OnTimer_Credits ()
{
	nCreditsFrame++;

	if (nCreditsFrame < 2000)
		SetTimer (Wnd->GetHWnd(), TIMER_CREDITS, CREDIT_DELAY, NULL);
	else
	{
		KillTimer (Wnd->GetHWnd(), TIMER_CREDITS);
		GoToMainMenu (true);
	}

	OnPaint ();
}

void OnTimer_TimeAttack()
{
	KillTimer(Wnd->GetHWnd(), TIMER_TIMEATTACK);

	if (nTimeCountDown)
	{
		nTimeCountDown--;
		if (!nTimeCountDown)
			PlayMusic(nLevel);
	}
	else
	{
		nTimeLimit--;
		if (!nTimeLimit)
		{
			GoToGameOverScreen();
			return;
		}
	}
	SetTimer(Wnd->GetHWnd(), TIMER_TIMEATTACK, 1000, NULL);
	OnPaint();
}

void StartAnimatedMove (Move m)
{
	fAnimationPercentage = 0.0f;
	AnimatedMove = m;
	SetTimer (Wnd->GetHWnd(), TIMER_ANIMATEDMOVE, nAnimatedMoveDelay, NULL);
	OnPaint ();
}

void PlayMusic(int level)
{
	if (Music)
	{
		Music->Stop();
		delete Music;
	}

	// Are we playing "Time Attack"?
	if (nTimeLimit > 0)
		switch (level)
		{
		case 1: Music = new CMP3File("Music\\Time Attack\\60.mp3"); break;
		case 2: Music = new CMP3File("Music\\Time Attack\\50.mp3"); break;
		case 3: Music = new CMP3File("Music\\Time Attack\\40.mp3"); break;
		case 4: Music = new CMP3File("Music\\Time Attack\\30.mp3"); break;
		case 5: Music = new CMP3File("Music\\Time Attack\\20.mp3"); break;
		case 6: Music = new CMP3File("Music\\Time Attack\\15.mp3"); break;
		default: Music = new CMP3File("Music\\Time Attack\\10.mp3"); break;
		}

	else
	{
		switch (level)
		{
		case MUSIC_CREDITS: Music = new CMP3File("Music\\Credits.mp3"); break;
		case MUSIC_TITLESCREEN: Music = new CMP3File("Music\\Title.mp3"); break;
		case 1: Music = new CMP3File("Music\\Level1.mp3"); break;
		case 2: Music = new CMP3File("Music\\Level2.mp3"); break;
		case 3: Music = new CMP3File("Music\\Level3.mp3"); break;
		case 4: Music = new CMP3File("Music\\Level4.mp3"); break;
		case 5: Music = new CMP3File("Music\\Level5.mp3"); break;
		case 6: Music = new CMP3File("Music\\Level6.mp3"); break;
		default: Music = new CMP3File("Music\\Level7.mp3"); break;
		}
	}
	
	Music->SetVolume(bMusic ? 0 : -9000);
	Music->SetMessageHandler(Wnd->GetHWnd());
	Music->Play();	
}

void RandomScream ()
{
	if (!bSound)
		return;

	int x = rand() % 7;
	LPSTR s;
	switch (x)
	{
	case 0: s = ".\\Sounds\\scream1.wav"; break;
	case 1: s = ".\\Sounds\\scream2.wav"; break;
	case 2: s = ".\\Sounds\\scream3.wav"; break;
	case 3: s = ".\\Sounds\\scream4.wav"; break;
	case 4: s = ".\\Sounds\\scream5.wav"; break;
	case 5: s = ".\\Sounds\\scream6.wav"; break;
	default: s = ".\\Sounds\\scream7.wav"; break;
	}

	PlaySound (s, NULL, SND_ASYNC | SND_FILENAME | SND_NODEFAULT);
}

void ResetBoard ()
{
	if (GameState)
		delete GameState;
	GameState = new CGameState ();
	UnselectAllPieces ();
}

void StopMusic()
{
	if (!Music)
		return;

	Music->Stop();
}

int WINAPI WinMain (HINSTANCE hInst, HINSTANCE, LPSTR, int nCmdShow)
{
	Wnd = new CMainWindow (hInst, 800, 600, NULL, WndProc);
	Display->CreateWindowedDisplay (Wnd->GetHWnd(), 800, 600);
	MainMenu = new CMainMenu(Display);
	LoadGoatAndTigerBMPs ();
	Background = new CBackgroundSurface (Display, BACKGROUND_TITLESCREEN);
	srand (timeGetTime ());
	OnKeyDown = OnKeyDown_DoNothing;
	OnLeftButtonDown = OnLeftButtonDown_MainMenu;
	OnLeftButtonUp = OnLeftButtonDown_DoNothing;
	Wnd->Show (true);
	PlayMusic(MUSIC_TITLESCREEN);
	PlaySound (NULL, NULL, SND_ASYNC | SND_NODEFAULT);

	OnPaint ();
	MSG msg;
	while (GetMessage (&msg, 0, 0, 0))
	{
		TranslateMessage (&msg);
		DispatchMessage (&msg);
	}

	return msg.wParam;
}