/* Copyright (c) Daniel Newman 2006
 *
 * e-mail: danielnewman106@hotmail.com
 */

#include "MP3File.h"

CMP3File::CMP3File()
{
	CreateCOMObjects();
}

CMP3File::CMP3File(char *szFileName)
{
	CreateCOMObjects();
	LoadFile(szFileName);
}

#define RELEASE(x) if (x) { x->Release(); x = NULL; }
CMP3File::~CMP3File()
{
	RELEASE(m_pMediaPosition)
	RELEASE(m_pMediaEvent)
	RELEASE(m_pMediaControl)
	RELEASE(m_pGraphBuilder)

	CoUninitialize();
}

void CMP3File::CreateCOMObjects()
{
	CoInitialize(NULL);

	/* Create the COM objects */
	if (FAILED(CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER,
	IID_IGraphBuilder,(void**)&m_pGraphBuilder)))
		return;
	m_pGraphBuilder->QueryInterface(IID_IMediaControl, (void**)&m_pMediaControl);
	m_pGraphBuilder->QueryInterface(IID_IMediaEvent, (void**)&m_pMediaEvent);
	m_pGraphBuilder->QueryInterface(IID_IMediaPosition, (void**)&m_pMediaPosition);
}

HRESULT CMP3File::LoadFile(char *szFileName)
{
	/* Convert to a 'wide' string */
	WCHAR wFileName[MAX_PATH];
	MultiByteToWideChar(CP_ACP, 0, szFileName, -1, wFileName, MAX_PATH);

	return m_pGraphBuilder->RenderFile(wFileName, NULL);
}

HRESULT CMP3File::Play()
{
	return m_pMediaControl->Run();
}

HRESULT CMP3File::Rewind()
{
	return m_pMediaPosition->put_CurrentPosition(0);
}

HRESULT CMP3File::SetMessageHandler(HWND hWnd)
{
	HRESULT hr = m_pMediaEvent->SetNotifyWindow((OAHWND)hWnd, WM_MP3FILEEVENT, 0);
	if (FAILED(hr))
		return hr;

	return m_pMediaEvent->SetNotifyFlags(0);
}

HRESULT CMP3File::SetVolume(long lVolume)
{
	IBasicAudio *pBasicAudio = NULL;
	HRESULT hr = m_pGraphBuilder->QueryInterface(IID_IBasicAudio, (void**)&pBasicAudio);

	if (FAILED(hr))
		return hr;

	hr = pBasicAudio->put_Volume(lVolume);
	
	pBasicAudio->Release();

	return hr;
}

HRESULT CMP3File::Stop()
{
	return m_pMediaControl->Stop();
}