//-----------------------------------------------------------------------------
// File: ddutil.cpp
//
// Desc: Routines for loading bitmap and palettes from resources
//
// Copyright (C) 1998-2001 Microsoft Corporation. All Rights Reserved.
//-----------------------------------------------------------------------------
#ifndef DDUTIL_H
#define DDUTIL_H

#include <ddraw.h>
#include <d3d.h>




//-----------------------------------------------------------------------------
// Classes defined in this header file 
//-----------------------------------------------------------------------------
class CDisplay;




//-----------------------------------------------------------------------------
// Flags for the CDisplay and CSurface methods
//-----------------------------------------------------------------------------
#define DSURFACELOCK_READ
#define DSURFACELOCK_WRITE




//-----------------------------------------------------------------------------
// Name: class CDisplay
// Desc: Class to handle all DDraw aspects of a display, including creation of
//       front and back buffers, creating offscreen surfaces and palettes,
//       and blitting surface and displaying bitmaps.
//-----------------------------------------------------------------------------
class CDisplay
{
protected:
    LPDIRECTDRAW        m_pDD;
    LPDIRECTDRAWSURFACE m_pddsFrontBuffer;
    LPDIRECTDRAWSURFACE m_pddsBackBuffer;

    HWND                 m_hWnd;
    RECT                 m_rcWindow;
    BOOL                 m_bWindowed;

public:
    CDisplay();
    ~CDisplay();

    // Access functions
    HWND                 GetHWnd()           { return m_hWnd; }
    LPDIRECTDRAW         GetDirectDraw()     { return m_pDD; }
    LPDIRECTDRAWSURFACE	 GetFrontBuffer()    { return m_pddsFrontBuffer; }
    LPDIRECTDRAWSURFACE  GetBackBuffer()     { return m_pddsBackBuffer; }

    // Status functions
    BOOL    IsWindowed()                     { return m_bWindowed; }

    // Creation/destruction methods
    HRESULT CreateFullScreenDisplay( HWND hWnd, DWORD dwWidth, DWORD dwHeight,
		                             DWORD dwBPP );
    HRESULT CreateWindowedDisplay( HWND hWnd, DWORD dwWidth, DWORD dwHeight );
	HRESULT CreateDDSurface (LPDIRECTDRAWSURFACE *lpSurf, DWORD dwWidth, DWORD dwHeight);
    HRESULT InitClipper();
    HRESULT UpdateBounds();
    virtual HRESULT DestroyObjects();

    // Display methods
    HRESULT Clear( DWORD dwColor = 0L );
    HRESULT Blt( DWORD x, DWORD y, LPDIRECTDRAWSURFACE pdds,
		         RECT* prc=NULL, DWORD dwFlags=0 );
    HRESULT Present();
};



#endif // DDUTIL_H

